@@ -20,12 +20,15 @@ class Downloader:
                    ("AudioDB", "_get_audiodb_artist_artwork_uri",
                     "_get_audiodb_album_artwork_uri",
                     "_get_audiodb_artist_info"),
-                   ("Deezer", "_get_deezer_artist_artwork_uri",
-                    "_get_deezer_album_artwork_uri", None),
+                   ("FanartTV", "_get_fanarttv_artist_artwork_uri",
+                    "_get_fanarttv_album_artwork_uri",
+                    None),
                    ("Spotify", "_get_spotify_artist_artwork_uri",
                     "_get_spotify_album_artwork_uri", None),
                    ("Itunes", None,
                     "_get_itunes_album_artwork_uri", None),
+                   ("Deezer", "_get_deezer_artist_artwork_uri",
+                    "_get_deezer_album_artwork_uri", None),
                    ("Last.fm", None,  # Doesn't work anymore
                     "_get_lastfm_album_artwork_uri",
                     "_get_lastfm_artist_info"),
