@@ -15,64 +15,109 @@ from gi.repository import Gtk, Gdk, GLib, Gio, Gst
 from datetime import datetime
 from gettext import gettext as _
 
-from lollypop.define import App, ArtSize, ArtBehaviour, Type, ViewType
-from lollypop.controller_information import InformationController
-from lollypop.controller_playback import PlaybackController
-from lollypop.controller_progress import ProgressController
+from lollypop.define import App, ArtSize, ArtBehaviour, AdaptiveSize
+from lollypop.widgets_player_progress import ProgressPlayerWidget
+from lollypop.widgets_player_buttons import ButtonsPlayerWidget
+from lollypop.widgets_player_artwork import ArtworkPlayerWidget
+from lollypop.widgets_player_label import LabelPlayerWidget
+from lollypop.objects_radio import Radio
 from lollypop.container import Container
 from lollypop.adaptive import AdaptiveWindow
 from lollypop.logger import Logger
+from lollypop.utils import emit_signal
+from lollypop.helper_signals import SignalsHelper, signals_map
 
 
-class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
-                 PlaybackController, ProgressController):
+class FullScreen(Gtk.Window, AdaptiveWindow, SignalsHelper):
     """
         Show a fullscreen window showing current track context
     """
 
-    def __init__(self, app):
+    @signals_map
+    def __init__(self):
         """
-            Init window for app
-            @param app as Gio.Application
+            Init window
         """
         Gtk.Window.__init__(self)
         AdaptiveWindow.__init__(self)
+        return [
+                (App().player, "current-changed", "_on_current_changed")
+        ]
+
+    def delayed_init(self):
+        """
+            Delay real init to get App().window == self
+        """
         self.get_style_context().add_class("black")
         self.set_title("Lollypop")
         self.__allocation = Gdk.Rectangle()
-        PlaybackController.__init__(self)
-        ProgressController.__init__(self)
-        self.set_application(app)
+        self.set_application(App())
         self.__timeout_id = None
         self.__signal1_id = self.__signal2_id = None
+        self.__background_id = None
         self.set_decorated(False)
-
-        builder = Gtk.Builder()
-        builder.add_from_resource("/org/gnome/Lollypop/FullScreen.ui")
-        builder.connect_signals(self)
-
         # Calculate cover size
         screen = Gdk.Screen.get_default()
         monitor = screen.get_monitor_at_window(App().main_window.get_window())
         geometry = screen.get_monitor_geometry(monitor)
-        self.__overlay_grid = builder.get_object("overlay_grid")
         art_size_fs = ArtSize.FULLSCREEN / self.get_scale_factor()
-        font_size_fs = 11 / self.get_scale_factor()
+        font_size_fs = 40 / self.get_scale_factor()
         if geometry.width > geometry.height:
-            self.__art_size = int(art_size_fs * geometry.height / 1080)
-            self.__font_size = int(font_size_fs * geometry.height / 1080)
+            art_size = int(art_size_fs * geometry.height / 1080)
+            font_size = int(font_size_fs * geometry.height / 1080)
         else:
-            self.__art_size = int(art_size_fs * geometry.width / 1920)
-            self.__font_size = int(font_size_fs * geometry.width / 1920)
+            art_size = int(art_size_fs * geometry.width / 1920)
+            font_size = int(font_size_fs * geometry.width / 1920)
+        builder = Gtk.Builder()
+        builder.add_from_resource("/org/gnome/Lollypop/FullScreen.ui")
+        builder.connect_signals(self)
+        self.__progress_widget = ProgressPlayerWidget()
+        self.__progress_widget.show()
+        self.__progress_widget.set_property("halign", Gtk.Align.CENTER)
+        self.__progress_widget.set_size_request(500, -1)
+        self.__buttons_widget = ButtonsPlayerWidget(["menu-button",
+                                                     "black-transparent"])
+        self.__buttons_widget.show()
+        self.__buttons_widget.set_size_request(500, -1)
+        self.__buttons_widget.set_property("valign", Gtk.Align.CENTER)
+        self.__buttons_widget.set_property("halign", Gtk.Align.CENTER)
+        self.__artwork_widget = ArtworkPlayerWidget()
+        self.__artwork_widget.show()
+        self.__artwork_widget.set_vexpand(True)
+        self.__artwork_widget.set_art_size(art_size, art_size)
+        self.__artwork_widget.set_property("valign", Gtk.Align.CENTER)
+        self.__artwork_widget.set_property("halign", Gtk.Align.CENTER)
+        self.__label_widget = LabelPlayerWidget(True, font_size)
+        self.__label_widget.show()
+        self.__label_widget.set_hexpand(True)
+        self.__label_widget.set_vexpand(True)
+        self.__label_widget.set_justify(Gtk.Justification.CENTER)
+        eventbox = Gtk.EventBox.new()
+        eventbox.show()
+        eventbox.connect("button-release-event",
+                         self.__on_image_button_release_event)
+        eventbox.connect("realize", self.__on_image_realize)
+        eventbox.add(self.__artwork_widget)
+        self.__revealer = builder.get_object("revealer")
+        self.__datetime = builder.get_object("datetime")
+        self.__overlay_grid = builder.get_object("overlay_grid")
         widget = builder.get_object("widget")
-        self._play_button = builder.get_object("play_btn")
-        self._next_button = builder.get_object("next_btn")
-        self._prev_button = builder.get_object("prev_btn")
-        if App().player.current_track.id == Type.RADIOS:
-            self._next_button.hide()
-            self._prev_button.hide()
-        self._play_image = builder.get_object("play_image")
-        self._pause_image = builder.get_object("pause_image")
+        if App().settings.get_value("artist-artwork"):
+            self.__overlay_grid.attach(self.__buttons_widget, 0, 4, 2, 1)
+            self.__overlay_grid.attach(self.__label_widget, 0, 2, 2, 1)
+            self.__overlay_grid.attach(self.__progress_widget, 0, 3, 2, 1)
+            self.__overlay_grid.attach(eventbox, 2, 2, 1, 3)
+            self.__progress_widget.set_margin_start(50)
+            eventbox.set_margin_end(50)
+            self.__label_widget.set_margin_start(50)
+            self.__label_widget.set_property("valign", Gtk.Align.END)
+            self.__artwork_widget.set_property("valign", Gtk.Align.END)
+        else:
+            self.__overlay_grid.attach(self.__buttons_widget, 0, 4, 3, 1)
+            self.__overlay_grid.attach(self.__label_widget, 0, 2, 3, 1)
+            self.__overlay_grid.attach(self.__progress_widget, 0, 3, 3, 1)
+            self.__overlay_grid.attach(eventbox, 0, 1, 3, 1)
+            eventbox.set_vexpand(True)
         close_btn = builder.get_object("close_btn")
         preferences = Gio.Settings.new("org.gnome.desktop.wm.preferences")
         layout = preferences.get_value("button-layout").get_string()
@@ -83,17 +128,7 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
             self.__overlay_grid.attach(close_btn, 2, 0, 1, 1)
             close_btn.set_property("halign", Gtk.Align.END)
         self._artwork = builder.get_object("cover")
-        self.__setup_controller()
-        self._title_label = builder.get_object("title")
-        self._artist_label = builder.get_object("artist")
-        self._album_label = builder.get_object("album")
-        self.__revealer = builder.get_object("revealer")
-        self._datetime = builder.get_object("datetime")
-        self._progress = builder.get_object("progress_scale")
-        self._timelabel = builder.get_object("playback")
-        self._total_time_label = builder.get_object("duration")
         self.connect("key-release-event", self.__on_key_release_event)
-
         # Add a navigation widget on the right
         self.__back_button = Gtk.Button.new_from_icon_name(
             "go-previous-symbolic", Gtk.IconSize.BUTTON)
@@ -108,8 +143,7 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
         self.__back_button.set_margin_bottom(5)
         self.__back_button.show()
         self.__background_artwork = builder.get_object("background_artwork")
-        self.__container = Container(ViewType.NOT_ADAPTIVE)
-        self.set_stack(self.__container.stack)
+        self.__container = Container()
         self.__container.show()
         self.__sidebar = Gtk.Grid()
         self.__sidebar.set_size_request(400, -1)
@@ -119,10 +153,10 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
         self.__sidebar.add(self.__back_button)
         self.__sidebar.add(self.__container)
         self.__sidebar.set_size_request(450, -1)
-        self.connect("can-go-back-changed", self.__on_can_go_back_changed)
-        self.connect("show-can-go-back", self.__on_show_can_go_back)
-        self.connect("size-allocate", self.__on_size_allocate)
         self.__sidebar.get_style_context().add_class("background-opacity")
+        self.__container.connect("can-go-back-changed",
+                                 self.__on_can_go_back_changed)
+        self.connect("size-allocate", self.__on_size_allocate)
         self.__revealer.add(self.__sidebar)
         self.add(widget)
 
@@ -130,15 +164,11 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
         """
             Setup window for current screen
         """
-        show_sidebar = App().settings.get_value("show-sidebar")
-        self.__container.show_sidebar(show_sidebar)
-        self.__signal1_id = App().player.connect("current-changed",
-                                                 self.on_current_changed)
-        self.__signal2_id = App().player.connect("status-changed",
-                                                 self.on_status_changed)
-        self.on_status_changed(App().player)
-        self.on_current_changed(App().player)
         Gtk.Window.do_show(self)
+        self.__buttons_widget.update()
+        self.__label_widget.update()
+        self.__progress_widget.update()
+        self.__setup_artwork_type()
         if self.__timeout_id is None:
             try:
                 interface = Gio.Settings.new("org.gnome.desktop.interface")
@@ -149,11 +179,12 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
             self.__timeout_id = GLib.timeout_add(1000,
                                                  self.__update_datetime,
                                                  show_seconds)
-        self.update_position(App().player.position / Gst.SECOND)
+        self.__progress_widget.update_position(
+            App().player.position / Gst.SECOND)
         screen = Gdk.Screen.get_default()
         monitor = screen.get_monitor_at_window(App().main_window.get_window())
         self.fullscreen_on_monitor(screen, monitor)
-
+        self.__container.setup_lists()
         # Disable screensaver (idle)
         App().inhibitor.manual_inhibit(
                 Gtk.ApplicationInhibitFlags.IDLE |
@@ -164,44 +195,10 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
             Clean window
         """
         Gtk.Window.do_hide(self)
-        if self.__signal1_id is not None:
-            App().player.disconnect(self.__signal1_id)
-            self.__signal1_id = None
-        if self.__signal2_id is not None:
-            App().player.disconnect(self.__signal2_id)
-            self.__signal2_id = None
         if self.__timeout_id is not None:
             GLib.source_remove(self.__timeout_id)
             self.__timeout_id = None
         App().inhibitor.manual_uninhibit()
-        ProgressController.on_destroy(self)
-
-    def on_status_changed(self, player):
-        """
-            Update controller
-            @param player as Player
-        """
-        ProgressController.on_status_changed(self, player)
-        PlaybackController.on_status_changed(self, player)
-
-    def on_current_changed(self, player):
-        """
-            Update controllers
-            @param player as Player
-        """
-        InformationController.on_current_changed(self,
-                                                 self.__art_size,
-                                                 self.__font_size)
-        ProgressController.on_current_changed(self, player)
-        self.__update_background()
-        if player.current_track.id is not None:
-            album_name = player.current_track.album.name
-            if player.current_track.year:
-                album_name += " (%s)" % player.current_track.year
-            self._album_label.set_markup(
-                "<span font='%s'>%s</span>" %
-                                        (self.__font_size - 1,
-                                         GLib.markup_escape_text(album_name)))
 
     @property
     def miniplayer(self):
@@ -215,6 +212,10 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
     def is_adaptive(self):
         return True
 
+    @property
+    def adaptive_size(self):
+        return AdaptiveSize.PHONE
+
     @property
     def container(self):
         """
@@ -226,23 +227,6 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
 #######################
 # PROTECTED           #
 #######################
-    def _on_artwork(self, surface):
-        """
-            Set artwork
-            @param surface as str
-        """
-        if surface is None:
-            if App().player.current_track.id == Type.RADIOS:
-                icon_name = "audio-input-microphone-symbolic"
-            else:
-                icon_name = "folder-music-symbolic"
-            self._artwork.set_from_icon_name(icon_name,
-                                             Gtk.IconSize.DIALOG)
-            self._artwork.set_size_request(self.__art_size, self.__art_size)
-        else:
-            InformationController._on_artwork(self, surface)
-            self._artwork.set_size_request(-1, -1)
-
     def _on_close_button_clicked(self, button):
         """
             Destroy self
@@ -260,12 +244,112 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
             button.get_image().set_from_icon_name("pan-start-symbolic",
                                                   Gtk.IconSize.BUTTON)
         else:
-            self.set_adaptive_stack(True)
             self.__revealer.set_reveal_child(True)
+            emit_signal(self, "adaptive-changed", True)
             button.get_image().set_from_icon_name("pan-end-symbolic",
                                                   Gtk.IconSize.BUTTON)
 
-    def _on_image_realize(self, eventbox):
+    def _on_current_changed(self, player):
+        """
+            Update background
+            @param player as Player
+        """
+        self.__update_background()
+
+#######################
+# PRIVATE             #
+#######################
+    def __setup_artwork_type(self):
+        """
+            Setup artwork type
+        """
+        fs_type = App().settings.get_value("fullscreen-type").get_int32()
+        context = self.__artwork_widget.get_style_context()
+        behaviour = ArtBehaviour.CROP_SQUARE
+        if fs_type & ArtBehaviour.ROUNDED:
+            context.add_class("image-rotate")
+            context.remove_class("small-cover-frame")
+            behaviour |= ArtBehaviour.ROUNDED
+        else:
+            context.remove_class("image-rotate")
+            context.add_class("small-cover-frame")
+        self.__artwork_widget.set_behaviour(behaviour)
+        self.__artwork_widget.update()
+
+    def __update_background(self, album_artwork=False):
+        """
+            Update window background
+            @param album_artwork as bool
+        """
+        try:
+            if App().player.current_track.id is None:
+                return
+            allocation = self.get_allocation()
+            if allocation.width <= 1 or allocation.height <= 1:
+                return
+            behaviour = App().settings.get_value("fullscreen-type").get_int32()
+            behaviour |= (ArtBehaviour.CROP | ArtBehaviour.DARKER)
+            # We don't want this for background, stored for album cover
+            behaviour &= ~ArtBehaviour.ROUNDED
+            if isinstance(App().player.current_track, Radio):
+                if self.__background_id == App().player.current_track.name:
+                    return
+                App().art_helper.set_radio_artwork(
+                                    App().player.current_track.name,
+                                    allocation.width,
+                                    allocation.height,
+                                    self.get_scale_factor(),
+                                    behaviour | ArtBehaviour.BLUR_MAX,
+                                    self.__on_artwork,
+                                    False)
+            elif not album_artwork and\
+                    App().settings.get_value("artist-artwork"):
+                if App().player.current_track.album.artists:
+                    artist = App().player.current_track.album.artists[0]
+                else:
+                    artist = App().player.current_track.artists[0]
+                if self.__background_id == artist:
+                    return
+                App().art_helper.set_artist_artwork(
+                                    artist,
+                                    allocation.width,
+                                    allocation.height,
+                                    self.get_scale_factor(),
+                                    behaviour,
+                                    self.__on_artwork,
+                                    False)
+            else:
+                if self.__background_id == App().player.current_track.album.id:
+                    return
+                App().art_helper.set_album_artwork(
+                                    App().player.current_track.album,
+                                    allocation.width,
+                                    allocation.height,
+                                    self.get_scale_factor(),
+                                    ArtBehaviour.BLUR_MAX |
+                                    ArtBehaviour.CROP |
+                                    ArtBehaviour.DARKER,
+                                    self.__on_artwork,
+                                    True)
+        except Exception as e:
+            Logger.error("Fullscreen::__update_background(): %s", e)
+
+    def __update_datetime(self, show_seconds=False):
+        """
+            Update datetime in headerbar
+            @param show_seconds as bool
+        """
+        now = datetime.now()
+        if show_seconds:
+            self.__datetime.set_label(now.strftime("%a %d %b %X"))
+        else:
+            self.__datetime.set_label(now.strftime("%a %d %b %X")[:-3])
+        if self.__timeout_id is None:
+            self.__timeout_id = GLib.timeout_add(60000, self.__update_datetime)
+            return False
+        return True
+
+    def __on_image_realize(self, eventbox):
         """
             Set cursor
             @param eventbox as Gtk.EventBox
@@ -275,7 +359,7 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
         except:
             Logger.warning(_("You are using a broken cursor theme!"))
 
-    def _on_image_button_release_event(self, widget, event):
+    def __on_image_button_release_event(self, widget, event):
         """
             Change artwork type
             @param widget as Gtk.Widget
@@ -296,95 +380,8 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
             fs_type = ArtBehaviour.NONE
         App().settings.set_value("fullscreen-type",
                                  GLib.Variant("i", fs_type))
-        self.__setup_controller()
+        self.__setup_artwork_type()
         self.__update_background()
-        InformationController.on_current_changed(self,
-                                                 self.__art_size,
-                                                 self.__font_size)
-
-#######################
-# PRIVATE             #
-#######################
-    def __setup_controller(self):
-        """
-            Setup controller
-        """
-        fs_type = App().settings.get_value("fullscreen-type").get_int32()
-        context = self._artwork.get_style_context()
-        behaviour = ArtBehaviour.CROP_SQUARE
-        if fs_type & ArtBehaviour.ROUNDED:
-            context.add_class("image-rotate")
-            context.remove_class("cover-frame")
-            behaviour |= ArtBehaviour.ROUNDED
-        else:
-            context.remove_class("image-rotate")
-            context.add_class("cover-frame")
-        InformationController.__init__(self, False, behaviour)
-
-    def __update_background(self, album_artwork=False):
-        """
-            Update window background
-            @param album_artwork as bool
-        """
-        try:
-            if App().player.current_track.id is None:
-                return
-            allocation = self.get_allocation()
-            if allocation.width <= 1 or allocation.height <= 1:
-                return
-            behaviour = App().settings.get_value("fullscreen-type").get_int32()
-            behaviour |= (ArtBehaviour.CROP | ArtBehaviour.DARKER)
-            # We don't want this for background, stored for album cover
-            behaviour &= ~ArtBehaviour.ROUNDED
-            if App().player.current_track.id == Type.RADIOS:
-                App().art_helper.set_radio_artwork(
-                                    App().player.current_track.radio_name,
-                                    allocation.width,
-                                    allocation.height,
-                                    self.get_scale_factor(),
-                                    behaviour | ArtBehaviour.BLUR_MAX,
-                                    self.__on_artwork,
-                                    False)
-            elif not album_artwork and\
-                    App().settings.get_value("artist-artwork"):
-                if App().player.current_track.album.artists:
-                    artist = App().player.current_track.album.artists[0]
-                else:
-                    artist = App().player.current_track.artists[0]
-                App().art_helper.set_artist_artwork(
-                                    artist,
-                                    allocation.width,
-                                    allocation.height,
-                                    self.get_scale_factor(),
-                                    behaviour,
-                                    self.__on_artwork,
-                                    False)
-            else:
-                App().art_helper.set_album_artwork(
-                                    App().player.current_track.album,
-                                    allocation.width,
-                                    allocation.height,
-                                    self.get_scale_factor(),
-                                    behaviour | ArtBehaviour.BLUR_MAX,
-                                    self.__on_artwork,
-                                    True)
-        except Exception as e:
-            Logger.error("Fullscreen::__update_background(): %s", e)
-
-    def __update_datetime(self, show_seconds=False):
-        """
-            Update datetime in headerbar
-            @param show_seconds as bool
-        """
-        now = datetime.now()
-        if show_seconds:
-            self._datetime.set_label(now.strftime("%a %d %b %X"))
-        else:
-            self._datetime.set_label(now.strftime("%a %d %b %X")[:-3])
-        if self.__timeout_id is None:
-            self.__timeout_id = GLib.timeout_add(60000, self.__update_datetime)
-            return False
-        return True
 
     def __on_artwork(self, surface, album_artwork):
         """
@@ -411,18 +408,7 @@ class FullScreen(Gtk.Window, AdaptiveWindow, InformationController,
             Go back in container stack
             @param button as Gtk.Button
         """
-        self.go_back()
-
-    def __on_show_can_go_back(self, window, back):
-        """
-            Show back button
-            @param window as Gtk.Window
-            @param back as bool
-        """
-        if back:
-            self.__back_button.show()
-        else:
-            self.__back_button.hide()
+        self.__container.go_back()
 
     def __on_can_go_back_changed(self, window, back):
         """
