@@ -16,8 +16,8 @@ import cairo
 from random import shuffle
 
 from lollypop.define import App, Type
-from lollypop.objects import Album
-from lollypop.utils import get_round_surface
+from lollypop.objects_album import Album
+from lollypop.utils import get_round_surface, emit_signal
 from lollypop.widgets_flowbox_rounded import RoundedFlowBoxWidget
 
 
@@ -46,8 +46,6 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
         """
             Populate widget content
         """
-        self.__album_ids = self._get_album_ids()
-        shuffle(self.__album_ids)
         RoundedFlowBoxWidget.populate(self)
         self._artwork.get_style_context().add_class("light-background")
 
@@ -65,14 +63,27 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
             Set artwork
         """
         RoundedFlowBoxWidget.set_artwork(self)
-        App().task_helper.run(self._create_surface)
+        if App().art.artwork_exists_in_cache(self.artwork_name,
+                                             "ROUNDED",
+                                             self._art_size,
+                                             self._art_size):
+            App().task_helper.run(
+                App().art.get_artwork_from_cache,
+                self.artwork_name, "ROUNDED",
+                self._art_size, self._art_size,
+                callback=(self.__on_load_from_cache,))
+        else:
+            self.__album_ids = self._get_album_ids()
+            shuffle(self.__album_ids)
+            App().task_helper.run(self._create_surface, True)
 
 #######################
 # PROTECTED           #
 #######################
-    def _create_surface(self):
+    def _create_surface(self, set_surface):
         """
             Get artwork surface
+            @param set_surface as bool
             @return cairo.Surface
         """
         surface = cairo.ImageSurface(cairo.FORMAT_RGB24,
@@ -84,7 +95,7 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
         ctx.fill()
         album_ids = list(self.__album_ids)
         positions = [(0, 0), (1, 0), (0, 1), (1, 1)]
-        self.__draw_surface(surface, ctx, positions, album_ids)
+        self.__draw_surface(surface, ctx, positions, album_ids, set_surface)
 
 #######################
 # PRIVATE             #
@@ -97,16 +108,20 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
         if self.__cancellable.is_cancelled():
             return
         self._artwork.set_from_surface(
-            get_round_surface(surface, self._scale_factor, 50))
-        self.emit("populated")
+            get_round_surface(surface, self._scale_factor, self._art_size / 4))
+        App().art.add_artwork_to_cache(self.artwork_name,
+                                       surface,
+                                       "ROUNDED")
+        emit_signal(self, "populated")
 
-    def __draw_surface(self, surface, ctx, positions, album_ids):
+    def __draw_surface(self, surface, ctx, positions, album_ids, set_surface):
         """
             Draw surface for first available album
             @param surface as cairo.Surface
             @param ctx as Cairo.context
             @param positions as {}
             @param album_ids as [int]
+            @param set_surface as bool
             @thread safe
         """
         # Workaround Gdk not being thread safe
@@ -122,7 +137,8 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
             ctx.set_source_surface(subsurface, 0, 0)
             ctx.paint()
             ctx.translate(-x, -y)
-            self.__draw_surface(surface, ctx, positions, album_ids)
+            self.__draw_surface(surface, ctx, positions,
+                                album_ids, set_surface)
         if self.__cancellable.is_cancelled():
             return
         elif album_ids and len(positions) > 0:
@@ -133,13 +149,26 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
                                                  self._scale_factor)
             if pixbuf is None:
                 GLib.idle_add(self.__draw_surface, surface,
-                              ctx, positions, album_ids)
+                              ctx, positions, album_ids, set_surface)
             else:
                 GLib.idle_add(draw_pixbuf, surface,
                               ctx, pixbuf, positions, album_ids)
         else:
             GLib.idle_add(self.__set_surface, surface)
 
+    def __on_load_from_cache(self, pixbuf):
+        """
+            Set artwork surface
+            @param pixbuf as GdkPixbuf.Pixbuf
+        """
+        if not self.__cancellable.is_cancelled() and pixbuf is not None:
+            surface = Gdk.cairo_surface_create_from_pixbuf(
+                pixbuf, self._artwork.get_scale_factor(), None)
+            self._artwork.set_from_surface(
+                    get_round_surface(surface, self._scale_factor,
+                                      self._art_size / 4))
+        emit_signal(self, "populated")
+
     def __on_unmap(self, widget):
         """
             Cancel drawing
