@@ -14,9 +14,9 @@ from gi.repository import Gtk
 
 from random import shuffle
 
-from lollypop.define import App, SelectionListMask, ArtBehaviour, Type
-from lollypop.utils import get_icon_name
-from lollypop.objects import Album
+from lollypop.define import App, ArtBehaviour
+from lollypop.utils import get_icon_name, emit_signal
+from lollypop.objects_album import Album
 from lollypop.widgets_flowbox_rounded import RoundedFlowBoxWidget
 
 
@@ -43,23 +43,9 @@ class RoundedArtistWidget(RoundedFlowBoxWidget):
         if self._artwork is None:
             RoundedFlowBoxWidget.populate(self)
             self.connect("destroy", self.__on_destroy)
-            self.connect("button-release-event",
-                         self.__on_button_release_event)
-            self.__gesture = Gtk.GestureLongPress.new(self)
-            self.__gesture.connect("pressed", self.__on_gesture_pressed)
-            # We want to get release event after gesture
-            self.__gesture.set_propagation_phase(Gtk.PropagationPhase.CAPTURE)
-            self.__gesture.set_button(0)
         else:
             self.set_artwork()
 
-    def show_overlay(self, show):
-        """
-            Show overlay
-            @param show as bool
-        """
-        pass
-
     def set_artwork(self):
         """
             Set artist artwork
@@ -75,14 +61,6 @@ class RoundedArtistWidget(RoundedFlowBoxWidget):
         self.set_size_request(self._art_size,
                               self._art_size + self.__font_height)
 
-    @property
-    def is_overlay(self):
-        """
-            True if widget is overlayed
-            @return bool
-        """
-        return False
-
 #######################
 # PROTECTED           #
 #######################
@@ -93,7 +71,7 @@ class RoundedArtistWidget(RoundedFlowBoxWidget):
         def set_icon_name():
             icon_name = get_icon_name(self._data) or "avatar-default-symbolic"
             self._artwork.set_from_icon_name(icon_name, Gtk.IconSize.DIALOG)
-            self.emit("populated")
+            emit_signal(self, "populated")
             self._artwork.get_style_context().add_class("artwork-icon-large")
 
         if self._artwork is None:
@@ -130,24 +108,6 @@ class RoundedArtistWidget(RoundedFlowBoxWidget):
 #######################
 # PRIVATE             #
 #######################
-    def __popup_menu(self, widget):
-        """
-            Popup menu for track
-            @param widget as Gtk.Widget
-        """
-        if App().settings.get_value("show-sidebar"):
-            return
-        from lollypop.view_artists_rounded import RoundedArtistsView
-        from lollypop.menu_selectionlist import SelectionListMenu
-        from lollypop.widgets_utils import Popover
-        menu = SelectionListMenu(self.get_ancestor(RoundedArtistsView),
-                                 self.data,
-                                 SelectionListMask.ARTISTS_VIEW)
-        popover = Popover()
-        popover.bind_model(menu, None)
-        popover.set_relative_to(widget)
-        popover.popup()
-
     def __on_artist_artwork(self, surface):
         """
             Finish widget initialisation
@@ -156,38 +116,13 @@ class RoundedArtistWidget(RoundedFlowBoxWidget):
         if self._artwork is None:
             return
         if surface is None:
+            self._artwork.get_style_context().add_class("artwork-icon")
             self._artwork.set_from_icon_name("avatar-default-symbolic",
                                              Gtk.IconSize.DIALOG)
         else:
+            self._artwork.get_style_context().remove_class("artwork-icon")
             self._artwork.set_from_surface(surface)
-        if self._artwork.props.surface is None:
-            self._artwork.get_style_context().add_class("artwork-icon")
-        self.emit("populated")
-
-    def __on_gesture_pressed(self, gesture, x, y):
-        """
-            Show current track menu
-            @param gesture as Gtk.GestureLongPress
-            @param x as float
-            @param y as float
-        """
-        self.__popup_menu(self)
-
-    def __on_button_release_event(self, widget, event):
-        """
-            Handle button release event
-            @param widget as Gtk.Widget
-            @param event as Gdk.Event
-        """
-        if event.button == 1:
-            # User clicked on random, clear cached one
-            if self._data == Type.RANDOMS:
-                App().albums.clear_cached_randoms()
-                App().tracks.clear_cached_randoms()
-            self.activate()
-        elif event.button == 3:
-            self.__popup_menu(self)
-        return True
+        emit_signal(self, "populated")
 
     def __on_destroy(self, widget):
         """
