@@ -15,7 +15,7 @@ from gi.repository import Gtk, GLib
 from gettext import gettext as _
 
 from lollypop.define import App, Type
-from lollypop.objects import Track
+from lollypop.objects_track import Track
 
 
 class LovedWidget(Gtk.Bin):
@@ -38,7 +38,6 @@ class LovedWidget(Gtk.Bin):
         builder.connect_signals(self)
         self.__artwork = builder.get_object("artwork")
         self.add(builder.get_object("widget"))
-        self.set_property("valign", Gtk.Align.CENTER)
         self.__set_artwork(self.__object.loved)
 
     def set_icon_size(self, icon_size):
